#include "stdafx.h"

#include "..\..\uelib2\uelib.h"
#include "resource.h"

using namespace uelib;

class AzDialog : public Dialog
{
public:
	AzDialog(size_t, size_t, DWORD);
	
public:
	AzDialog operator++(){
		do_increment(); return *this;
	}
	void do_increment();
	void do_stop();
	DWORD id(){ return _threadid; }

private:
	void onInit();
	void onDestroy();
	bool onOK();
	bool onCancel();
//	bool onCommand( UINT cmd, UINT id, HWND ctrl );
//	bool onMessage( UINT msg, WPARAM wp, LPARAM lp );

private:
	size_t _size;
	size_t _totalsize;
	size_t _unitsize;
	HWND   _hstatic;
	DWORD  _threadid;
	bool   _stop;
};

inline AzDialog::AzDialog(size_t tsize, size_t usize, DWORD id)
	: Dialog(IDD_DIALOG1)
	, _size(0)
	, _totalsize(tsize), _unitsize(usize)
	, _threadid(id), _stop(false)
{
}

inline void AzDialog::onInit()
{
	_hstatic = ::GetDlgItem( getHwnd(),
		IDC_STATIC1 );

	TCHAR tmp[256]={0};

	wsprintf( tmp, _T("waiting... "),
			_size );

	::SetWindowText( _hstatic, tmp );
}

inline void AzDialog::onDestroy()
{
	Dialog::onDestroy();
}

inline bool AzDialog::onOK()
{
	return true;
}
inline bool AzDialog::onCancel()
{
	return true;
}

inline void AzDialog::do_increment()
{
	TCHAR tmp[256]={0};
	_size += _unitsize;

	if( _stop ) return;

	if( _totalsize==0 )
	{
		wsprintf( tmp, _T("read %7d byte"),
			_size );
	}
	else
	{
		if( _size > _totalsize ) _size = _totalsize;
		int per = _totalsize==0 ? 0 :
			100*_size/_totalsize;
		wsprintf( tmp, _T(" %7d/%7d %3d%%"),
			_size, _totalsize, per );
	}
	::SetWindowText( _hstatic, tmp );
}

inline void AzDialog::do_stop()
{
	::SetWindowText( _hstatic, _T("I...") );
	_stop = true;
}

//--------------------------------------------------------
typedef bool (*AZCLOSEDATA)(HWND, DWORD, bool);
#define WM_HTMLPARSEEND WM_USER+301

class AzDLDialog : public Dialog
{
public:
	AzDLDialog(size_t, size_t, DWORD,
		HMODULE, bool, const String&);

public:
	AzDLDialog operator++(){
		do_increment(); return *this;
	}
	void do_increment();
	void do_stop();
	DWORD id(){ return _threadid; }
	bool viewflag(){ return _viewflag; }
	LPCTSTR fname(){ return _fname.c_str(); }

private:
	void onInit();
	void onDestroy();
	bool onOK();
	bool onCancel();
	bool onCommand( UINT cmd, UINT id, HWND ctrl );
//	bool onMessage( UINT msg, WPARAM wp, LPARAM lp );

private:
	size_t _size;
	size_t _totalsize;
	size_t _unitsize;
	HWND   _hstatic1, _hstatic2;
	HWND   _hprogress;
	DWORD  _threadid;
	bool   _viewflag;
	bool   _stop;
	const String _fname;
	HMODULE _hDLL;
};

inline AzDLDialog::AzDLDialog(size_t tsize, size_t usize, DWORD id,
					   HMODULE hDLL,
					   bool viewflag=false, 
					   const String& fname=String(_T("")) )
	: Dialog(IDD_DIALOG2)
	, _size(0)
	, _totalsize(tsize), _unitsize(usize)
	, _threadid(id)
	, _viewflag(viewflag), _fname(fname)
{
}

inline void AzDLDialog::onInit()
{
	_hstatic1 = ::GetDlgItem( getHwnd(),
		IDC_STATIC1 );
	_hstatic2 = ::GetDlgItem( getHwnd(),
		IDC_STATIC2 );
	_hprogress = ::GetDlgItem( getHwnd(),
		IDC_PROGRESS1 );

	TCHAR tmp[256]={0};
	if( _totalsize==0 )
		wsprintf( tmp, _T("read %7d byte"),
			_size );
	else
		wsprintf( tmp, _T(" %7d/%7d 0%%"),
			_size, _totalsize );

	::SetWindowText( _hstatic1, _fname.c_str() );
	::SetWindowText( _hstatic2, tmp );

	if( _totalsize!=0 )
	{
		::SendMessage( _hprogress, PBM_SETRANGE, 0, 
			MAKELPARAM(0, _totalsize/_unitsize) );
		::SendMessage( _hprogress, PBM_SETSTEP, 1, 0L );
	}
}

inline void AzDLDialog::onDestroy()
{
	Dialog::onDestroy();
}

inline bool AzDLDialog::onOK()
{
	return true;
}
inline bool AzDLDialog::onCancel()
{
	return true;
}

inline bool AzDLDialog::onCommand( UINT cmd, UINT id, HWND ctrl )
{
	if( id==IDC_BUTTON1 )
	{
		do_stop(); update();

		PostMessage( ::GetParent( getHwnd() ),
			WM_HTMLPARSEEND, 1, 0L );
	}
	return false;
}

inline void AzDLDialog::do_increment()
{
	TCHAR tmp[256]={0};
	_size += _unitsize;

	if( _totalsize==0 )
	{
		wsprintf( tmp, _T("read %7d byte"),
			_size );
	}
	else
	{
		if( _size > _totalsize ) _size = _totalsize;
		int per = _totalsize==0 ? 0 :
			100*_size/_totalsize;
		wsprintf( tmp, _T(" %7d/%7d %3d%%"),
			_size, _totalsize, per );
		::SendMessage( _hprogress, PBM_STEPIT, 0, 0L );
	}
	::SetWindowText( _hstatic2, tmp );
}

inline void AzDLDialog::do_stop()
{
	::SetWindowText( _hstatic2, _T("I...") );
}

//--------------------------------------------------------

class AzTxtDialog : public Dialog
{
public:
	AzTxtDialog(const TCHAR* uri,
			map<String, String>& m,
			const String& iniTxt=String(_T("")) );

private:
	void onInit();
	void onDestroy();
	bool onOK();
	bool onCancel();
	bool onCommand( UINT cmd, UINT id, HWND ctrl );
//	bool onMessage( UINT msg, WPARAM wp, LPARAM lp );

private:
	const String         _uri, _txt;
	map<String, String>& _m;
	HWND                 _hEdit;
};

inline AzTxtDialog::AzTxtDialog(const TCHAR* uri,
			map<String, String>& m,
			const String& initTxt )
	: Dialog(IDD_DIALOG3)
	, _uri(uri), _txt( initTxt ), _m(m)
{
}

inline void AzTxtDialog::onInit()
{
	_hEdit = ::GetDlgItem( getHwnd(), IDC_EDIT1 );
	if( !_txt.empty() )
		::SetWindowText( _hEdit, _txt.c_str() );
	::SetFocus( _hEdit );
}

inline void AzTxtDialog::onDestroy()
{
	Dialog::onDestroy();
}

inline bool AzTxtDialog::onOK()
{
	// eLXg̒𒲂ׂ
	int len = ::SendMessage( _hEdit, WM_GETTEXTLENGTH, 0, 0L );

	// obt@mۂ
	TCHAR *t = new TCHAR[len+2];

	// eLXgĂ
	::GetWindowText( _hEdit, t, len+1 );
	String s = t;
	delete [] t;

	// ύX
	map<String, String>::iterator itr;
	itr = _m.insert( _m.begin(), pair<String, String>( _uri, s ) );

	if( s != itr->second )
		_m[ _uri ] = s;
	return true;
}

inline bool AzTxtDialog::onCancel()
{
	return true;
}

inline bool AzTxtDialog::onCommand( UINT cmd, UINT id, HWND ctrl )
{
	return false;
}

//--------------------------------------------------------

class AzTxtDialog2 : public Dialog
{
public:
	AzTxtDialog2(const TCHAR* title);

public:
	const String& getText();

private:
	void onInit();
	void onDestroy();
	bool onOK();
	bool onCancel();
//	bool onCommand( UINT cmd, UINT id, HWND ctrl );
//	bool onMessage( UINT msg, WPARAM wp, LPARAM lp );

private:
	String _title, _txt;
	HWND   _hEdit;
};

inline AzTxtDialog2::AzTxtDialog2(const TCHAR* title)
	: Dialog(IDD_DIALOG3)
	, _title( title )
{}

inline void AzTxtDialog2::onInit()
{
	_hEdit = ::GetDlgItem( getHwnd(), IDC_EDIT1 );
	::SetFocus( _hEdit );
	::SetWindowText( getHwnd(), _title.c_str() );
}

inline void AzTxtDialog2::onDestroy()
{
	Dialog::onDestroy();
}

inline bool AzTxtDialog2::onOK()
{
	// eLXg̒𒲂ׂ
	int len = ::SendMessage( _hEdit, WM_GETTEXTLENGTH, 0, 0L );

	// obt@mۂ
	TCHAR *t = new TCHAR[len+2];

	// eLXgĂ
	::GetWindowText( _hEdit, t, len+1 );
	_txt = t;
	delete [] t;

	return true;
}

inline bool AzTxtDialog2::onCancel()
{
	return true;
}

inline const String& AzTxtDialog2::getText()
{
	return _txt;
}

