
#include "ntpedit.h"
#include "caret.h"

using namespace ntpedit;


Scroll::Scroll(PaintTool& t, HWND h)
:_tool(t), _vwnd(h), _x(0), _y(0)
{
	_siv.cbSize = sizeof(SCROLLINFO);
	_siv.fMask  = SIF_DISABLENOSCROLL|SIF_POS|SIF_RANGE;
	_siv.nMin   = _siv.nMax = 0;
	_siv.nPage  = _siv.nPos = 0;
	_siv.nTrackPos = 0;

	_sih.cbSize = sizeof(SCROLLINFO);
	_sih.fMask  = SIF_DISABLENOSCROLL|SIF_POS|SIF_RANGE;
	_sih.nMin   = _sih.nMax = 0;
	_sih.nPage  = _sih.nPos = 0;
	_sih.nTrackPos = 0;

	setV(); setH();
}

void Scroll::reset()
{
	_siv.fMask = SIF_DISABLENOSCROLL|SIF_POS|SIF_RANGE;
	_siv.nMin   = _siv.nMax = 0;
	_siv.nPage  = _siv.nPos = 0;
	_siv.nTrackPos = 0;

	_sih.fMask  = SIF_DISABLENOSCROLL|SIF_POS|SIF_RANGE;
	_sih.nMin   = _sih.nMax = 0;
	_sih.nPage  = _sih.nPos = 0;
	_sih.nTrackPos = 0;

	_x = _y = 0;
	setV(); setH(); update();
}

void Scroll::update()
{
	::InvalidateRect( _vwnd, NULL, FALSE );
	::UpdateWindow( _vwnd );
}

void Scroll::onSize( int vlines, int vtotal, int hlines, int htotal )
{
	// size = ̑s
	// line = `s
	// h_size = ̍ő啶
	// h_line = ̕`敶
	if( vlines==0 && vtotal==0 ) return;

	if( _siv.nMax == vtotal && _siv.nPage == (UINT)vlines &&
		hlines==0 && htotal==0 )
		return;

	_siv.fMask = _sih.fMask
		= SIF_DISABLENOSCROLL|SIF_POS|SIF_RANGE|SIF_PAGE;
	_siv.nMax   = vtotal;
	_siv.nPage  = vlines;

	_sih.nMax   = htotal;
	_sih.nPage  = hlines;

	setV(); setH(); update();
}

int Scroll::onVScroll( short code, short pos )
{
	int y=0;

	switch ( code ) {
	case SB_LINEUP:
		y = -1;
		break;
	case SB_LINEDOWN:
		y = 1;
		break;
	case SB_PAGEUP:
		y = -1 * _siv.nPage;
		break;
	case SB_PAGEDOWN:
		y = _siv.nPage;
		break;
	case SB_THUMBTRACK:
		y = pos - _siv.nPos;
		break;
	default:
		y = 0;
		return 0;
//		break;
	}

	y = max(-1 * _siv.nPos, min(y, _siv.nMax - _siv.nPos));
	if( y != 0 ) {
		// XN[o[L->ƂA
		// ړłȂȂ邩RgAEg
		// łAXN[o[ŉsȂ̂ɁAʂXN[
		// Ă܂̂hKvAB
//		if( _siv.nMax < _siv.nPos + (int)_siv.nPage + y )
//			return 0;
		if( _siv.nMax < _siv.nPos + (int)_siv.nPage + y &&
			_siv.nMax > (int)_siv.nPage )
			return 0;
		if( _siv.nMax < (int)_siv.nPage && _y==0 ) return 0;

		_y += y;
		_siv.nPos += y;
		setV();
		update();
	}
/*
#ifdef DEBUG
	TCHAR tmp[64];
	wsprintf(tmp, _T("nPos = %d\n"), _siv.nPos );
	OutputDebugString(tmp);
#endif
*/
	return y;
}

void Scroll::onHScroll( short code )
{
	int oldx = _x;

	_x += code*_tool.getHScrollUnitSize();

	if( code!=0 ) update();
}

void Scroll::onKeyDown( int code )
{
	WPARAM wp;
	if( code==VK_PRIOR )
		wp = MAKEWPARAM(SB_PAGEUP,0);
	else if( code==VK_NEXT )
		wp = MAKEWPARAM(SB_PAGEDOWN,0);
	else
		return;

	::PostMessage( _vwnd, WM_VSCROLL,
		wp, 0L );
}

