
#include "ntpedit.h"

#include "view.h"
#include "caret.h"
#include "painter.h"
#include "basepainter.h"

#define WM_TEXTUPDATED  WM_USER+310
#define WM_SHOWPOPUP    WM_USER+311

extern void setMessage(LPCTSTR m);


using namespace ntpedit;

View::View( Doc& d, HWND wnd )
	:Window( _T("NTP2 VIEW"), WS_CHILD|WS_VISIBLE|WS_VSCROLL/*|WS_HSCROLL*/ )
	,_doc( d )
	,_parent( wnd ), _wndHandler(NULL) // <- ȂRgAEgH
{
	static bool ClassRegistered = false;
	if( !ClassRegistered )
	{
		// \ẑ݁ANXo^s
		ClassRegistered = true;
		WNDCLASS wc      = {0};
		wc.lpszClassName = _className;
		wc.style         = CS_DBLCLKS;
		wc.hCursor       = ::LoadCursor( NULL, IDC_IBEAM );
		registerWnd( &wc );
	}

	// 쐬
	create( NULL, wnd );
	show();
}

View::~View()
{
	destroy();
}

void View::replacePainter(BasePainter* p)
{
	_p->Delete();
	_p = p;
}

void View::deletePainter()
{ _p->Delete(); }

void View::onCreate()
{
	_tool   = auto_ptr<PaintTool>( new PaintTool(_doc, _hwnd) );
	_scroll = auto_ptr<Scroll>   ( new Scroll(*_tool, _hwnd) );
	_caret  = auto_ptr<Caret>    ( new Caret(_doc, *_tool, _hwnd) );
//	_p      = auto_ptr<BasePainter> ( new Painter(_doc, *_tool, _hwnd) );
	_p      = new Painter(_doc, *_tool, _hwnd);
}

void View::onDestroy()
{
	Window::onDestroy();
}

bool View::onCommand(UINT id, HWND ctrl)
{
	return false;
}

LRESULT View::onMessage( UINT msg, WPARAM wp, LPARAM lp )
{
	switch( msg )
	{
	case WM_PAINT:{
		PAINTSTRUCT ps;
		DWORD t[2]; TCHAR temp[64];
		::BeginPaint( getHwnd(), &ps );
		_caret->hide();
		// HideCaret  caret ͊mɏ邪A
		// hԂƂĕ`悵Ă킯ł
		// Ȃ炵B
		// łƁA
		// 肽Ȃ񂾂ȁ[B
		// ܁AԂɗ]T΁A
		// Lbĝs`̂zˁB
		_caret->onPaint();
		t[0] = ::GetTickCount();
		_p->onPaint( &ps, _scroll->getVStartPos(), 
			_scroll->getHScrollSize() );
		t[1] = ::GetTickCount();
		_caret->show();
		wsprintf(temp, _T("onPaint : %d (msec)"), t[1]-t[0]);
//		setMessage(temp);
		::EndPaint( getHwnd(), &ps );
		}return Window::onMessage( msg, wp, lp );
	case WM_SIZE:
		if( wp != SIZE_MINIMIZED ){
			// Caret::paste ƂɁAWM_SIZE  wp=0L 
			// Ă̂ŁAp΍AB
			if( lp != 0L )
				_p->onSize( HIWORD(lp), LOWORD(lp) );
			_scroll->onSize( _tool->getLines(),
				_tool->getTotalLines() );
		}
#ifdef DEBUG
		{
		TCHAR tmp[64];
		wsprintf( tmp, _T("View : wp=%d : lp=0x%x\n"), wp, lp );
		OutputDebugString( tmp );
		}
#endif
		break;
	case WM_VSCROLL:
		{
//		static int count=0;
//		TCHAR temp[64];
//		wsprintf(temp, _T("onVScroll:%d times"), count++);
//		setMessage(temp);
		int y;
		y = _scroll->onVScroll( (int)LOWORD(wp), (short)HIWORD(wp) );
		_caret->onVScroll( y );
		}
		break;
	case WM_HSCROLL:
//		_scroll->onHScroll( (int)LOWORD(wp) );
		break;
	case WM_CHAR:
//	case WM_IME_CHAR:
		if( _wndHandler!=NULL )
			::PostMessage( _wndHandler, WM_TEXTUPDATED, 0, 0L );
		_caret->onChar(wp);
		_scroll->onSize( _tool->getLines(),
			_tool->getTotalLines() );
		break;
	case WM_IME_COMPOSITION:
		if(lp & GCS_RESULTSTR)
		{
			if( _wndHandler!=NULL )
				::PostMessage( _wndHandler, WM_TEXTUPDATED, 0, 0L );
			_caret->onIMEComposition(lp);
			_scroll->onSize( _tool->getLines(),
				_tool->getTotalLines() );
			break;
		}
		else
		{
			return Window::onMessage( msg, wp, lp );
		}
		break;
	case WM_IME_NOTIFY:
		_caret->onIMENotify(wp);
		break;
	case WM_KEYDOWN:
		if( wp==VK_RIGHT || wp==VK_LEFT||
		    wp==VK_DOWN || wp==VK_UP || wp==0x10 )
		{
			int flag = _caret->onKeyDown( (int)wp, _scroll->getHScrollSize() );
			_scroll->onVScroll( flag&0x0000FFFF, 0 );
			_scroll->onHScroll( (flag&0xFFFF0000)>>16 );
		}
		else if( wp==VK_PRIOR || wp==VK_NEXT )
			_scroll->onKeyDown(wp);
		else if( wp==VK_HOME || wp==VK_END )
			_caret->onKeyDown2( (int)wp,
				_scroll->getVStartPos() );
		break;
	case WM_KEYUP:
		if( wp==0x10 ) _caret->onKeyUp(wp);
		break;
	case WM_SETFOCUS:
		_caret->onSetFocus();
		break;
	case WM_KILLFOCUS:
		_caret->onKillFocus();
		break;
	case WM_LBUTTONDOWN:
		_caret->onLButtonDown( LOWORD(lp), HIWORD(lp),
			_scroll->getVStartPos(),
			_scroll->getHScrollSize() );
		break;
	case WM_LBUTTONUP:
		_caret->onLButtonUp( LOWORD(lp), HIWORD(lp),
			_scroll->getVStartPos() );
		_p->onLButtonUp( LOWORD(lp), HIWORD(lp),
			_scroll->getVStartPos() );
		break;
	case WM_RBUTTONUP:
		if( _wndHandler != NULL )
			::PostMessage( _wndHandler, WM_SHOWPOPUP, wp, lp );
		break;
	case WM_MOUSEMOVE:
		_caret->onMouseMove( LOWORD(lp), HIWORD(lp),
			_scroll->getVStartPos(),
			_scroll->getHScrollSize() );
		break;
	case 0x020A: // WM_MOUSEWHEEL
		_caret->onMouseWheel( HIWORD(wp),LOWORD(wp) );
		break;

	default:
		return Window::onMessage( msg, wp, lp );
	}
	return 0;
}

