
#include "stdafx.h"
#include "t2.h"
#include "t2_2.h"

using namespace uelib;

namespace uelib{

class TxtCreator2
{
public:
	static Txt2* createTxt(int charset, 
		const UCHAR* base, const ULONG size)
	{
		switch(charset)
		{
		case SJIS:
			return new SjisTxt2(base, size); break;
		case EucJP:
			return new EucTxt2(base, size); break;
		case UTF8:
			return new Utf8Txt2(base, size, true); break;
		case UTF8N:
			return new Utf8Txt2(base, size, false); break;

		case UTF16b:    // Unicode  (UTF-16)  : BOML BE
			return new Utf16Txt2(base, size, true, 0); break;
		case UTF16l:    // Unicode  (UTF-16)  : BOML LE
			return new Utf16Txt2(base, size, true, 1); break;
		case UTF16BE:   // Unicode  (UTF-16BE): BOM
			return new Utf16Txt2(base, size, false, 0); break;
		case UTF16LE:   // Unicode  (UTF-16LE): BOM
			return new Utf16Txt2(base, size, false, 1); break;

		default:
			return new SjisTxt2(base, size);
		}
		return NULL;
	}
};

} //namespace uelib

//--------------------------------------------------------

TxtDecoder2::TxtDecoder2(int charset)
     :_charset(charset)
{}

TxtDecoder2::~TxtDecoder2()
{
	close();
}

bool TxtDecoder2::open(const TCHAR* fname)
{
	if( !_fp.open(fname) ) return false;

	// {IɉsR[h͎ʁB
	// 

	_t = auto_ptr<Txt2>( TxtCreator2::createTxt( _charset,
		_fp.base(), _fp.size() ) );
//	_t = auto_ptr<Txt2>( new Utf8Txt2(_fp.base(),
//		 	_fp.size(), true ));

	return true;
}

bool TxtDecoder2::open(const UCHAR *b, const ULONG s)
{
	_t = auto_ptr<Txt2>( 
		TxtCreator2::createTxt( _charset,
		b, s ) );
	return true;
}

void TxtDecoder2::close()
{
	_fp.close();
}

charset TxtDecoder2::autoDetect(const TCHAR* fname)
{
	FileR f;
	if(!f.open( fname )) return charset(0);

	charset cs=charset(0);
	// BOM check
	if( f.size()>4 )
		if(charset(0) != (cs = checkBOM( f.base() )) )
			return cs;

	// ǂɂYȂƂ肠 SJIS
	return SJIS;
//	return charset(0);
}

charset TxtDecoder2::autoDetect(const UCHAR* b,
								const ULONG s)
{
	charset cs=charset(0);
	// BOM check
	if( s > 4 )
		if(charset(0) != (cs = checkBOM(b)) )
			return cs;

	// g`FbN
	// ܂ utf8 `FbN
//	Txt* t = new Utf8Txt2(b, s, false);
	Utf8Txt2 t(b, s, false);
	size_t uppersize = s>=1024 ? 1024 : s;
	for(size_t pos=0; pos<uppersize; )
	{
		int pt = t.charNext();
		if( pt==-1 || pt==0 )
			break;
		else if( pt==3 ) return UTF8N;
		else         pos += pt;

	}
	if( pos >= uppersize ) return UTF8N;
	
	//  euc-sjis `FbN
	const UCHAR* p = b;
	for( size_t pos1=0; pos1<uppersize; p++ )
	{
		int pt = checkCharCode(p);
		if( pt >=4 && pt <= 15 ) return EucJP;
		if( pt >=2 && pt <= 3  ) return SJIS;
	}

	// ǂɂYȂƂ肠 SJIS
	return SJIS;
}

charset TxtDecoder2::checkBOM(const UCHAR* b)
{
	const UCHAR* p = b;
	//UTF-8 signature:      EF BB - BF
	//UTF-16(LE) signature: FF FE
	//UTF-16(BE) signature: FE FF
	//UTF-32(LE) signature: FF FE - 00 00
	//UTF-32(BE) signature: 00 00 - FE FF

	if( p[0] == 0xEF && p[1] == 0xBB 
		&& p[2] == 0xBF ) return UTF8;

	return charset(0);
}

/*
̎ʂ\l
Estr̐擪pJȉꍇA֐l1
Estr̐擪2VtgJISR[h̏ꍇA֐l2
Estr̐擪2EUCpJiR[h̏ꍇA֐l4
Estr̐擪2EUCSpR[h̏ꍇA֐l8
Estr̐擪3VJIS(X0208-1983)VtgR[h̏ꍇA
@֐l16
Estr̐擪3JIS(X0208-1978)VtgR[h̏ꍇA
@֐l32
Estr̐擪3JIS[}(X0201)VtgR[h̏ꍇA
@֐l64
Estr̐擪3ASCII VtgR[h̏ꍇA֐l128
ẼP[XɋNꍇ́A֐l̘͂_a
ÊłȂꍇ́A֐l0
*/
int TxtDecoder2::checkCharCode(const unsigned char* str)
{
	int val = 0;
	unsigned char b1, b2, b3;

	b1 = *str++;
	b2 = *str++;
	b3 = *str;
	if (b1 == 0x1b) {
		if (b2 == '$' && b3 == 'B') return 16;
		if (b2 == '$' && b3 == '@') return 32;
		if (b2 == '(' && b3 == 'J') return 64;
		if (b2 == '(' && b3 == 'B') return 128;
		return 0;
	}
	if ( b1 >= 0xa0 && b1 <= 0xdf) val |= 1;
	if ((b1 >= 0x81 && b1 <= 0x9f ||
		 b1 >= 0xe0 && b1 <= 0xfc) && 
		(b2 >= 0x40 && b2 <= 0xfc && b2 != 0x7f)) val |= 2;
	if (b1 == 0x8e && (b2 >= 0xa0 && b2 <= 0xdf)) val |= 4; 
	if ((b1 >= 0xa1 && b1 <= 0xfe) &&
		(b2 >= 0xa1 && b1 <= 0xfe)) val |= 8;

	return val;
}

//--------------------------------------------------------
