
#include "stdafx.h"
#include "txt.h"
#include "txt2.h"
#include "txt3.h"
#include "txt4.h"

using namespace uelib;

//--------------------------------------------------------

Txt* TxtCreator::createTxt(int charset, 
		const UCHAR* base, const ULONG size)
{
	switch(charset)
	{
	case SJIS:
		return new SjisTxt(base, size); break;
	case EucJP:
		return new EucTxt(base, size);  break;
	case IsoJP:
		return new JisTxt(base, size);  break;
	case UTF7:
		return new Utf7Txt(base, size); break;
	case UTF8:
		return new Utf8Txt(base, size, true); break;
	case UTF8N:
		return new Utf8Txt(base, size, false); break;
	case UTF16b:    // Unicode  (UTF-16)  : BOML BE
		return new Utf16Txt(base, size, true, 0); break;
	case UTF16l:    // Unicode  (UTF-16)  : BOML LE
		return new Utf16Txt(base, size, true, 1); break;
	case UTF16BE:   // Unicode  (UTF-16BE): BOM
		return new Utf16Txt(base, size, false, 0); break;
	case UTF16LE:   // Unicode  (UTF-16LE): BOM
		return new Utf16Txt(base, size, false, 1); break;
	case UTF32b:    // Unicode  (UTF-32)  : BOML BE
		return new Utf32Txt(base, size, true, 0); break;
	case UTF32l:    // Unicode  (UTF-32)  : BOML LE
		return new Utf32Txt(base, size, true, 1); break;

	default:
		return new SjisTxt(base, size);
	}
}

//--------------------------------------------------------

charset Txt::autoDetect(const TCHAR *fname, lbcode* lb)
{
	FileR f;
	if(!f.open( fname )) return charset(0);

	// ܂
	charset        cs = charset(0);
	size_t  uppersize = f.size()-5 > 1024 ? 1024 : f.size();

	// BOM check
	if( f.size()>4 )
		if(charset(0) != (cs = checkBOM(f)) )
			return cs;

	// UTF8 g`FbN
	Utf8Txt t(f.base(), uppersize, false);
	if( t.readtest() ) cs = UTF8N;

	// EUC - JIS - SJIS `FbN
	const UCHAR* p = f.base();
	for( size_t pos1=0; pos1<uppersize; p++, pos1++ )
	{
		int pt = checkCharCode(p);
		if( pt >=16            ){ cs = IsoJP; break; }
		if( pt >=4 && pt <= 15 ){ cs = EucJP; break; }
		if( pt >=2 && pt <= 3  ){ cs = SJIS;  break; }
	}

	// ǂɂYȂƂ肠 SJIS
	if( cs==charset(0) ) cs = SJIS;

	// LB `FbNvȂȂ炱 return
	if( lb==NULL ) return cs;

	// LineBreak `FbN
	*lb = checkMultiLB( f.base(), f.size() );

	return cs;
}

charset Txt::checkBOM(FileR& f)
{
	const UCHAR* p = reinterpret_cast<const UCHAR*>(f.base());
	//UTF-8 signature:      EF BB - BF
	//UTF-16(LE) signature: FF FE
	//UTF-16(BE) signature: FE FF
	//UTF-32(LE) signature: FF FE - 00 00
	//UTF-32(BE) signature: 00 00 - FE FF
	if( p[0]==0xff && p[1]==0xfe )
		// UTF-32LE
		if( p[2]==0x00 && p[3]==0x00 ) return UTF32l;
		// UTF-16LE
		else                           return UTF16l;
	// UTF-16BE
	else if( p[0]==0xfe && p[1]==0xff )return UTF16b;
	// UTF-8
	else if( p[0] == 0xEF && p[1] == 0xBB 
		&& p[2] == 0xBF )              return UTF8;
	// UTF-32BE
	else if( p[0] == 0x00 && p[1] == 0x00 &&
		p[2]==0xfe && p[3]==0xff)      return UTF32b;

	return charset(0);
}

Txt::lbcode Txt::checkMultiLB(const UCHAR *base, const ULONG size)
{
	UINT i=0;
	const UCHAR *p=base;
	const USHORT *ps;
	while( i < size )
	{
		// 2 byte
		if( i+1 > size )
		{
			ps = reinterpret_cast<const USHORT*>(&p[i]);
			if( *ps==0x000a || *ps==0x0a00 ) return LF;
			if( *ps==0x000d || *ps==0x0d00 )
			{
				ps = reinterpret_cast<const USHORT*>(&p[i+2]);
				if( *ps==0x000a || *ps==0x0a00 ) return CRLF;
				else                             return CR;
			}
		}

		// 1 byte
		if(p[i]=='\r'){
			if(p[i+1]=='\n') return CRLF;
			else             return CR;
		}else if(p[i]=='\n') return LF;

		i++;
	}
	return CRLF;
}

/*
̎ʂ\l
Estr̐擪pJȉꍇA֐l1
Estr̐擪2VtgJISR[h̏ꍇA֐l2
Estr̐擪2EUCpJiR[h̏ꍇA֐l4
Estr̐擪2EUCSpR[h̏ꍇA֐l8
Estr̐擪3VJIS(X0208-1983)VtgR[h̏ꍇA
@֐l16
Estr̐擪3JIS(X0208-1978)VtgR[h̏ꍇA
@֐l32
Estr̐擪3JIS[}(X0201)VtgR[h̏ꍇA
@֐l64
Estr̐擪3ASCII VtgR[h̏ꍇA֐l128
ẼP[XɋNꍇ́A֐l̘͂_a
ÊłȂꍇ́A֐l0
*/
int Txt::checkCharCode(const unsigned char* str)
{
	int val = 0;
	unsigned char b1, b2, b3;

	b1 = *str++;
	b2 = *str++;
	b3 = *str;
	if (b1 == 0x1b) {
		if (b2 == '$' && b3 == 'B') return 16;
		if (b2 == '$' && b3 == '@') return 32;
		if (b2 == '(' && b3 == 'J') return 64;
		if (b2 == '(' && b3 == 'B') return 128;
		return 0;
	}
	if ( b1 >= 0xa0 && b1 <= 0xdf) val |= 1;
	if ((b1 >= 0x81 && b1 <= 0x9f ||
		 b1 >= 0xe0 && b1 <= 0xfc) && 
		(b2 >= 0x40 && b2 <= 0xfc && b2 != 0x7f)) val |= 2;
	if (b1 == 0x8e && (b2 >= 0xa0 && b2 <= 0xdf)) val |= 4; 
	if ((b1 >= 0xa1 && b1 <= 0xfe) &&
		(b2 >= 0xa1 && b1 <= 0xfe)) val |= 8;

	return val;
}

//--------------------------------------------------------

TxtDecoder::TxtDecoder(int charset)
     :_charset(charset)
{}

TxtDecoder::~TxtDecoder()
{
	close();
}

bool TxtDecoder::open(const TCHAR* fname)
{
	if( !_fp.open(fname) ) return false;

	// {IɉsR[h͎ʁB
	// 

	_t = auto_ptr<Txt>( TxtCreator::createTxt( _charset,
		_fp.base(), _fp.size() ) );

	return true;
}

void TxtDecoder::close()
{
	_fp.close();
}

//--------------------------------------------------------

TxtEncoder::TxtEncoder(int charset, int linebreak)
     :_charset(charset),_lb(linebreak)
{}

TxtEncoder::~TxtEncoder()
{}

bool TxtEncoder::open(const TCHAR *fname)
{
	if( !_fp.open(fname) ) return false;

	_t = auto_ptr<Txt>( TxtCreator::createTxt( _charset,
						NULL, 0 ) );

	return true;
}

void TxtEncoder::close()
{
	_fp.close();
}

void TxtEncoder::writeLine(const String& s, 
						   bool firstline, bool lastline)
{
	// ŏ̍ŝƂBOMo
	if( firstline ) _t->writeBOM(_fp);

	_t->writeLine(_fp, s);

	// ŌɉsR[hB
	// AŏIsɂ͉s͂Ȃ
	if( !lastline ) _t->writeLB(_fp, _lb);
}

