
#ifndef _UELIB_TXT2_H_
#define _UELIB_TXT2_H_

#include "stdafx.h"
#include "txt.h"
#include "util.h"

// txt2.h
//  Shift-JIS, EUC, ISO-2022-JP ɂĂ
//  ̓IȓǂݏNX͂B

namespace uelib{


//----------------------------------------------------------------
class SjisTxt : public Txt
{
public:
	SjisTxt(const UCHAR* b, const ULONG s)
		:Txt(b, s){}

	String readLine()
    {
		// EOF ܂ŗ _state  0 ɂ
		const UCHAR *p = &_base[_pos], *pp=NULL;
		size_t start = _pos, length=0;
		while(1)
		{
			if( *p=='\n' )
			{
				_state = 2;
				break;
			}
			else if( *p=='\r' )
			{
				if( p[1]!='\n' )
				{ _state=2; break; }
			}
			else if( _pos>=_size ){
				_state = 0;
				break;
			}
			pp=(const UCHAR*)CharNextA((const char*)p);
			_pos   += (size_t)(pp - p);
			length += (size_t)(pp - p);
			p = pp;
		}
		if( _state == 2 ) _pos++;

		char *sp = new char [length+1];
		strncpy(sp, (const char*)&_base[start], length);
		sp[length] = '\0';
		if( sp[length-1]=='\r' ) sp[length-1] = '\0';
		String s = sp;
		delete [] sp;
		return s;
    }

	void writeLine(FileW& fp, const String& s)
	{
		string as = s.a_str();

		fp.write( as.c_str(), (ULONG)as.size() );
	}

};

//----------------------------------------------------------------
class Iso2022BaseTxt : public Txt
{
public:
	Iso2022BaseTxt(const UCHAR* b, const ULONG s)
		:Txt(b,s){}

public:
	String readLine()
	{
		// EOF ܂ŗ _state  0 ɂ
		const UCHAR *p = &_base[_pos];
		size_t start = _pos, length=0;

		// sT
		while(1)
		{
			if( *p=='\n' )
			{
				_state = 2; break;
			}
			else if( *p=='\r' )
			{
				if( p[1]!='\n' )
				{ _state=2; break; }
			}
			else if( _pos>=_size )
			{
				_state = 0; break;
			}
			p++, _pos++, length++;
		}
		if( _state == 2 ) _pos++;

		// s̃obt@mۂăRs[
		char *sp = new char [length+1];
		strncpy(sp, (const char*)&_base[start], length);
		sp[length] = '\0';
		if( sp[length-1]=='\r' ) sp[length-1] = '\0';

		// ISO -> SJIS
		char *ep = new char [length+1];
		iso2sjis(sp, ep);

		String s = ep;
		delete [] sp; delete [] ep;
		return s;
	}

	void writeLine(FileW& fp, const String& s)
	{
		string as = s.a_str();

		// SJIS -> ISO
		// EUC  JIS ƂŕKvȃoCgႤ̂ŁA
		// oCgKvς郁\bhKv
		char* ac = new char[ getWriteBufSize( as.c_str() )+1 ];
		sjis2iso( as.c_str(), ac );

		fp.write( ac, strlen(ac) );
		delete [] ac;
	}

private:
	// (EUC|JIS) -> SJIS
	virtual void iso2sjis(const char* in, char* out)=0;
	// SJIS -> (EUC|JIS)
	virtual void sjis2iso(const char* in, char* out)=0;
	// ݂ɕKvȃobt@TCYς
	virtual ULONG getWriteBufSize(const char* in) = 0;
};

//----------------------------------------------------------------
class EucTxt : public Iso2022BaseTxt
{
public:
	EucTxt(const UCHAR* b, const ULONG s)
		:Iso2022BaseTxt(b, s){}

private:
	// EUC -> SJIS
	void iso2sjis(const char* in, char* out)
	{
		// zϊB
		for(; *in!=NULL; )
		{
			// pȂȂɂȂ
			if( (UCHAR)*in <= 0x7f || in[1]==NULL )
			{ *out++ = *in++; continue; }

			// pJi
			if( (UCHAR)*in == 0x8e )
			{ *out++ = in[1]; in+=2; continue; }

			unsigned short s = e2s(in[0], in[1]);
			*out++ = (char)( s >> 8 );
			*out++ = (char)( s & 0x00ff );
			in+=2;
		}
		*out = '\0';
	}

	// EUC -> SJIS ̃Tu֐B
	unsigned short e2s(unsigned char hib, unsigned char lob)
	{
		// ꕶϊB
		if( hib==0x8e ) hib=0;
		else			hib=hib&~0x80, lob=lob&~0x80;

		lob += (hib & 1) ? 0x1f : 0x7d;
		if (lob >= 0x7f) lob++;
		hib = ((hib - 0x21) >> 1) + 0x81;
		if (hib > 0x9f) hib += 0x40;

		return (hib << 8) | lob;
	}

	// SJIS ->EUC
	void sjis2iso(const char* in, char* out)
	{
		for( ; *in!=NULL; )
		{
			// pJi
			if( (UCHAR)*in >= 0xa0 && (UCHAR)*in <= 0xdf )
			{ *out++ = (char)0x8e; *out++ = *in++; continue; }
			// p
			if( (UCHAR)*in <=0x7f || in[1]==NULL )
			{ *out++ = *in++; continue; }

			unsigned short e = s2e(in[0], in[1]);
			*out++ = (char)(e>>8);
			*out++ = (char)(e&0x00ff);
			in += 2;
		}
		*out = '\0';
	}

	// SJIS -> EUC ̃Tu֐B
	unsigned short s2e(unsigned char hib, unsigned char lob)
	{
		hib -= (hib <= 0x9f) ? 0x71 : 0xb1;
		hib = (hib << 1) + 1;
		if( lob >= 0x9e )
			lob -= 0x7e, hib++;
		else if( lob > 0x7f )
			lob -= 0x20;
		else
			lob -= 0x1f;

		hib |= 0x80, lob |= 0x80;

		return (hib << 8) | lob;
	}

	virtual ULONG getWriteBufSize(const char* in)
	{
		ULONG len = 0;
		for(; *in!='\0'; in++)
			if( (UCHAR)*in >= 0xa0 && (UCHAR)*in <= 0xdf )
				len+=2;
			else
				len++;
		return len;
	}
};

//----------------------------------------------------------------
class JisTxt : public Iso2022BaseTxt
{
public:
	JisTxt(const UCHAR* b, const ULONG s)
		:Iso2022BaseTxt(b, s), _cs(ASCII){}

private:
	enum codeset{ ASCII, JIS, KANA };

	// JIS -> SJIS
	virtual void iso2sjis(const char* in, char* out)
	{
		// zϊB
		for(; *in!=NULL; )
		{
			if( checkCodeSet(in) ){ in+=3; continue; }

			if( _cs==ASCII )
			{
				*out++ = *in++;
			}
			else if( _cs==KANA )
			{
				*out++ = *in++ + 0x80;
			}
			else
			{
				unsigned short s = j2s(in[0],in[1]);
				*out++ = (char)( s >> 8 );
				*out++ = (char)( s & 0x00ff );
				in+=2;
			}
		}
		*out = '\0';
	}

	// JIS -> SJIS ̃Tu֐B
	unsigned short j2s(unsigned char hib, unsigned char lob)
	{
		lob += (hib & 1) ? 0x1f : 0x7d;
		if( lob >= 0x7f ) lob++;
		hib = ((hib - 0x21) >> 1) + 0x81;
		if( hib > 0x9f ) hib += 0x40;

		return (hib << 8) | lob;
	}

	// SJIS -> JIS
	virtual void sjis2iso(const char* in, char* out)
	{
		for( ; *in!=NULL; )
		{
			// pJi
			if( (UCHAR)*in >= 0xa0 && (UCHAR)*in <= 0xdf )
			{
				if( _cs!=KANA ) 
					outputCodeSet(out, KANA), out+=3, _cs=KANA;
				 *out++ = *in++ - 0x80;
			}
			// p
			else if( (UCHAR)*in <=0x7f || in[1]==NULL )
			{
				if( _cs!=ASCII )
					outputCodeSet(out, ASCII), out+=3, _cs=ASCII;
				*out++ = *in++;
			}
			else
			{
				if( _cs!=JIS ) 
					outputCodeSet(out, JIS), out+=3, _cs=JIS;
				unsigned short j = s2j(in[0],in[1]);
				*out++ = (char)(j>>8);
				*out++ = (char)(j&0x00ff);
				in += 2;
			}
		}
		outputCodeSet(out, ASCII), out+=3, _cs=ASCII;
		*out = '\0';
	}

	// SJIS -> JIS ̃Tu֐B
	unsigned short s2j(unsigned char hib, unsigned char lob)
	{
		hib -= (hib <= 0x9f) ? 0x71 : 0xb1;
		hib  = (hib << 1) + 1;
		if( lob > 0x7f ) lob--;
		if( lob >= 0x9e )
			lob -= 0x7d, hib++;
		else
			lob -= 0x1f;

		return (hib << 8) | lob;
	}

	// CodeSet
	bool checkCodeSet(const char* p)
	{
		//
		if( p[0]!=0x1B ) return false;
		if( p[1]==0x24 && p[2]==0x42 )
		{
			_cs = JIS; return true;
		}
		else if( p[1]==0x28 )
		{
			if( p[2]==0x42 )
			{
				_cs = ASCII; return true;
			}
			else if( p[2]==0x49 )
			{
				_cs = KANA;  return true;
			}
		}
		return false;
	}

	// CodeSet o
	void outputCodeSet(char* p, codeset cs)
	{
		if( cs==ASCII )
		{
			*p++ = 0x1B; *p++ = 0x28; *p++ = 0x42;
		}
		else if( cs==KANA )
		{
			*p++ = 0x1B; *p++ = 0x28; *p++ = 0x49;
		}
		else if( cs==JIS )
		{
			*p++ = 0x1B; *p++ = 0x24; *p++ = 0x42;
		}
	}

	virtual ULONG getWriteBufSize(const char* in)
	{
		ULONG len = 3;
		codeset cs=ASCII;
		for(; *in!='\0';)
		{
			// pJi
			if( (UCHAR)*in >= 0xa0 && (UCHAR)*in <= 0xdf )
			{
				if( cs!=KANA ) len+=3, cs=KANA;
				len++, in++;
			}
			// p
			else if( (UCHAR)*in <=0x7f || in[1]==NULL )
			{
				if( cs!=ASCII ) len+=3, cs=ASCII;
				len++, in++;
			}
			// Sp(2byteKv)
			else
			{
				if( cs!=JIS ) len+=3, cs=JIS;
				len+=2, in+=2;
			}
		}
		return len;
	}
private:
	codeset _cs;
};

} //namespace uelib
#endif

