
#include "stdafx.h"
#include "window.h"
#include "cmdarg.h"


using namespace uelib;
//
// BaseWindow
////////////////////////////////////////
HINSTANCE BaseWindow::_hInst;
vector< Window* > Window::_vWnd;
vector< Dialog* > Dialog::_vDlg;

BaseWindow::BaseWindow()
	:_hwnd(NULL)
{
}

void BaseWindow::show()
{ 
	::ShowWindow(_hwnd, SW_SHOW);
	::UpdateWindow(_hwnd); 
}

void BaseWindow::hide()
{
	::ShowWindow(_hwnd, SW_HIDE);
	::UpdateWindow(_hwnd); 
}

void BaseWindow::msgLoop()
{
	for( MSG msg; ::GetMessage( &msg, NULL, 0, 0 ); )
	{
		if( !preTranslateMessage( &msg ) )
		{
			::TranslateMessage( &msg );
			::DispatchMessage( &msg );
		}
	}
}

//
// Window
////////////////////////////////////////
Window::Window( LPCTSTR className, DWORD style, DWORD styleEx )
	: _className( className )
	, _style( style )
	, _styleEx( styleEx )
{
}

Window::~Window()
{
	destroy();
}

bool Window::create(
	LPCTSTR wndName, HWND parent, 
	int x, int y, int w, int h, void* param )
{
	return (NULL != (_hwnd = ::CreateWindowEx(
		_styleEx, _className, wndName, _style,
		x, y, w, h, parent, NULL,
		BaseWindow::_hInst, &_atom //param
	)));
}

void Window::destroy()
{
	size_t size = _vWnd.size();

	if( _vWnd.empty() ) return;

	else if( size==1 ){
		_vWnd.clear(); return;
	}

    vector<Window*>::iterator   itr    = _vWnd.begin();
    vector<Window*>::iterator   itrEnd = _vWnd.end();

	// Y hwnd {Ċm
	for( ; itr != itrEnd ; itr++ ){
		// p
		if( (*itr)->getHwnd()==this->getHwnd() ){
			_vWnd.erase(itr);
			return;
		}
	}

}

bool Window::registerWnd(WNDCLASS* cls)
{
	cls->hInstance   = _hInst;
	cls->lpfnWndProc = baseProc;
	_atom = ::RegisterClass( cls );

	if(_atom==NULL) return false;

	_vWnd.push_back(this);
	return true;
}

bool Window::registerWnd(ATOM a)
{
	_atom = a;
	_vWnd.push_back(this);
	return true;
}

LRESULT CALLBACK Window::baseProc(
	HWND wnd, UINT msg, WPARAM wp, LPARAM lp )
{
	size_t size = _vWnd.size();

//	if( msg == WM_SIZE ) return ::DefWindowProc( wnd, msg, wp, lp );

	if( _vWnd.empty() ) ::DefWindowProc( wnd, msg, wp, lp );

	vector<Window*>::iterator  itr    = _vWnd.begin();
	vector<Window*>::iterator  itrEnd = _vWnd.end();

	if( msg==WM_CREATE )
	{
		// NULL Ȃ hwnd Ċm
		CREATESTRUCT* cs = reinterpret_cast<CREATESTRUCT*>(lp);
		for( ; itr != itrEnd ; itr++ ){
//			if( (*itr)->getHwnd()==NULL ){
			ATOM *a = reinterpret_cast<ATOM*>(cs->lpCreateParams);
			if( (*itr)->atom() == *a ){
				(*itr)->setHwnd(wnd); break;
			}
		}
	}else{
		// Y hwnd {Ċm
		for( ; itr != itrEnd ; itr++ ){
			if( (*itr)->getHwnd()==wnd ) break;
		}
	}
	if(itr == itrEnd)
		return ::DefWindowProc( wnd, msg, wp, lp );
	else
		return (*itr)->proc(msg, wp, lp);
}

LRESULT CALLBACK Window::proc(
	UINT msg, WPARAM wp, LPARAM lp )
{
	if( msg==WM_CREATE ){
		onCreate();
		return 0;

	}else if( msg==WM_COMMAND ){
		return onCommand(LOWORD(wp), (HWND)lp);

	}else if( msg==WM_CLOSE ){
		onDestroy();
//		return 0;
		return ::DefWindowProc( this->getHwnd(), msg, wp, lp );

	}else{
		return onMessage(msg, wp, lp);
	}

	return ::DefWindowProc( this->getHwnd(), msg, wp, lp );
}

void Window::onCreate()
{

}

void Window::onDestroy()
{
	::PostQuitMessage( 0 );
	destroy();
}

bool Window::onCommand( UINT id, HWND ctrl )
{
	return false;
}

LRESULT Window::onMessage( UINT msg, WPARAM wp, LPARAM lp )
{
	return ::DefWindowProc( this->getHwnd(), msg, wp, lp );
}

//
// Dialog
////////////////////////////////////////

Dialog::Dialog( UINT id )
	:_rID( id )
{
}

Dialog::~Dialog()
{
	// EChEjYĂ
	if( getHwnd() != NULL )
		end( IDCANCEL );
}

void Dialog::end( UINT code )
{
	_endCode = code;

	if( type() == MODAL )
		::EndDialog( getHwnd(), code );
	else
		::DestroyWindow( getHwnd() );

}

void Dialog::doModal( HWND parent )
{
	_type = MODAL;

	_vDlg.push_back(this);
	::DialogBoxParam( _hInst, MAKEINTRESOURCE(_rID), parent,
		(DLGPROC)proc, reinterpret_cast<LPARAM>(this) );
}

void Dialog::doModeless( HWND parent )
{
	_type = MODELESS;
	_vDlg.push_back(this);
	_hwnd = ::CreateDialogParam( _hInst, MAKEINTRESOURCE(_rID), parent,
		(DLGPROC)proc, reinterpret_cast<LPARAM>(this) );
}

BOOL CALLBACK Dialog::proc( HWND wnd, UINT msg, WPARAM wp, LPARAM lp )
{
	size_t size = _vDlg.size();

	vector<Dialog*>::iterator  itr    = _vDlg.begin();
	vector<Dialog*>::iterator  itrEnd = _vDlg.end();

	if( msg==WM_INITDIALOG )
	{
		// NULL Ȃ hwnd Ċm
		for( ; itr != itrEnd ; itr++ ){
			if( (*itr)->getHwnd()==NULL ){
				(*itr)->setHwnd(wnd); break;
			}
		}
	}else{
		// Y hwnd {Ċm
		for( ; itr != itrEnd ; itr++ ){
			if( (*itr)->getHwnd()==wnd ) break;
		}
	}
	if(itr == itrEnd)
		return FALSE;

	switch( msg )
	{
	case WM_INITDIALOG:
		(*itr)->onInit();
		return FALSE;

	case WM_DESTROY:
		(*itr)->onDestroy();
		(*itr)->setHwnd(NULL);
		_vDlg.erase(itr);
		if( _vDlg.empty() && Window::_vWnd.empty() )
			PostQuitMessage( 0 );
		return FALSE;

	case WM_COMMAND:
		switch( LOWORD(wp) ){
		case IDOK:
			if( (*itr)->onOK() )
				(*itr)->end( IDOK );
			return TRUE;

		case IDCANCEL:
			if( (*itr)->onCancel() )
				(*itr)->end( IDCANCEL );
			return TRUE;

		default:
			return (*itr)->onCommand( HIWORD(wp), LOWORD(wp),
				reinterpret_cast<HWND>(lp) ) ? TRUE : FALSE;
		}

	default:
		return (*itr)->onMessage( msg, wp, lp ) ? TRUE : FALSE;
	}
}

void Dialog::onInit()
{
	// Ȃ
}

void Dialog::onDestroy()
{
	// Ȃ
}

bool Dialog::onOK()
{
	// Ȃ
	return true;
}

bool Dialog::onCancel()
{
	// Ȃ
	return true;
}

bool Dialog::onCommand( UINT, UINT, HWND )
{
	// Ȃ
	return false;
}

bool Dialog::onMessage( UINT, WPARAM, LPARAM )
{
	// Ȃ
	return false;
}

//
// WinMain
////////////////////////////////////////
extern int ueMain();

// WinMain ֐͂BueMain  wrap B

#ifdef _WIN32_WCE

// CE p WinMain ֐
int WINAPI WinMain( HINSTANCE hInst, HINSTANCE hPInst,
					LPTSTR cmd, int n)
{
	BaseWindow::_hInst = hInst;
	Argv::setCmdLine( cmd );
	ueMain();
	return 0;
}

#else //_WIN32_WCE

// PC p WinMain ֐
int APIENTRY WinMain(HINSTANCE hInst, HINSTANCE hPInst,
			LPSTR cmd, int n)
{
	BaseWindow::_hInst = hInst;
	Argv::setCmdLine( cmd );
	ueMain();
	return 0;
}

#endif //_WIN32_WCE

