
#ifndef _UELIB_WINDOW_H_
#define _UELIB_WINDOW_H_

#include <vector>
#include <memory>

using namespace std;

namespace uelib {

// uelib2 ̖ړI
// AvP[V PC, CE ̈ႢӎȂ
// WinMain ֐̉B

//////////////////////////////////
// BaseWindow
//////////////////////////////////
class BaseWindow
{
public:
	BaseWindow();
//	virtual ~BaseWindow();

public:
	void show();
	void hide();
	void update() const;
	void msgLoop();
	HWND getHwnd() const;
	void setHwnd(HWND h){ _hwnd = h; }

	void setFocus();
	void moveTo( int l, int t, int r, int b );
	void getClientRect( RECT* rc ) const;
	virtual bool preTranslateMessage(MSG* m) = 0;

public:
	HWND _hwnd;
	static HINSTANCE _hInst;
};

inline void BaseWindow::update() const
{ ::UpdateWindow(_hwnd); }

inline HWND BaseWindow::getHwnd() const
{ return _hwnd; }

inline void BaseWindow::setFocus()
{ ::SetFocus( _hwnd );}

inline void BaseWindow::moveTo( int l, int t, int r, int b )
{
	::MoveWindow( _hwnd, l, t, r-l, b-t, TRUE ); 
}

inline void BaseWindow::getClientRect( RECT* rc ) const
	{ ::GetClientRect( _hwnd, rc ); }

//////////////////////////////////
// Window
//////////////////////////////////
class Window : public BaseWindow
{
public:
	Window( LPCTSTR className, DWORD style, DWORD styleEx=0L );
	virtual ~Window();

	//@{ EChE쐬 //@}
	bool create( LPCTSTR wndName=NULL,      HWND parent=NULL,
	             int  x     =CW_USEDEFAULT, int y      =CW_USEDEFAULT,
	             int  width =CW_USEDEFAULT, int height =CW_USEDEFAULT,
	             void* param=NULL );

	//@{ EChEj //@}
	void destroy();

public:
	bool registerWnd(WNDCLASS* cls);
	bool Window::registerWnd(ATOM a);
	ATOM atom(void) const;
	virtual bool preTranslateMessage(MSG* m)
	{ return false; }

public:
	static vector< Window* > _vWnd;

protected:
	virtual void    onCreate();
	virtual void    onDestroy();
	virtual bool    onCommand( UINT id, HWND ctrl );
	virtual LRESULT onMessage( UINT msg, WPARAM wp, LPARAM lp );

protected:
	LPCTSTR     _className;

private:
	static LRESULT CALLBACK baseProc( HWND, UINT, WPARAM, LPARAM );
	LRESULT CALLBACK proc( UINT, WPARAM, LPARAM );

private:
	const DWORD _style, _styleEx;
	ATOM _atom;
};

//--------------------------------------------
inline ATOM Window::atom() const
	{ return _atom; }

//////////////////////////////////
// Dialog
//////////////////////////////////
class Dialog : public BaseWindow
{
public:
	enum dlgType{ MODAL, MODELESS };

	Dialog( UINT id );
	~Dialog();
	void doModal( HWND parent=NULL );
	void doModeless( HWND parent=NULL );
	void end( UINT code );

public:
	virtual bool preTranslateMessage(MSG* m)
	{ return false; }
	dlgType type() const;
	UINT    endCode() const;

protected:

	virtual void onInit();
	virtual void onDestroy();
	virtual bool onOK();
	virtual bool onCancel();
	virtual bool onCommand( UINT cmd, UINT id, HWND ctrl );
	virtual bool onMessage( UINT msg, WPARAM wp, LPARAM lp );

	//@{ qACeIDHWNDϊ //@}
	HWND item( UINT id ) const;
	//@{ ACeɑ΂ăbZ[WM //@}
	LRESULT sendMsgToItem( UINT id, UINT msg, WPARAM wp=0, LPARAM lp=0 );


protected:
	static vector< Dialog* > _vDlg;

private:
	static BOOL CALLBACK proc( HWND, UINT, WPARAM, LPARAM );

private:
	dlgType    _type;
	UINT       _endCode;
	const UINT _rID;
};

inline Dialog::dlgType Dialog::type() const
{ return _type; }

inline UINT Dialog::endCode() const
{ return _endCode; }

inline HWND Dialog::item( UINT id ) const
{ return ::GetDlgItem( getHwnd(), id ); }

inline LRESULT Dialog::sendMsgToItem( UINT id, UINT msg, WPARAM wp, LPARAM lp )
{ return ::SendDlgItemMessage( getHwnd(), id, msg, wp, lp ); }

} //namespace uelib
#endif //_UELIB_WINDOW_H_
